namespace Printing
{
    using System;
    using System.Threading;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Printing;
    using System.Windows.Threading;

    public enum PrintMode
    {
        Default,
        Bitmap,
        Vector
    }

    public class PrinterTester
    {
        private Dispatcher _dispatcher;

        public PrinterTester(Dispatcher dispatcher)
        {
            _dispatcher = dispatcher;
        }

        public void Print(PrintMode mode)
        {
            var printDocument = new PrintDocument();

            printDocument.PrintPage += printDocument_PrintPage;

            switch (mode)
            {
                case PrintMode.Vector:
                    printDocument.Print("Test", new PrinterFallbackSettings { ForceVector = true, OpacityThreshold = 0 });
                    break;
                case PrintMode.Bitmap:
                    printDocument.PrintBitmap("Test");
                    break;
                default:
                    printDocument.Print("Test");
                    break;
            }
        }

        void printDocument_PrintPage(object sender, PrintPageEventArgs e)
        {
            DoInUiThread(() =>
            {
                var visualRoot = new StackPanel { HorizontalAlignment = HorizontalAlignment.Center };
                e.PageVisual = visualRoot;

                visualRoot.Children.Add(new TextBlock { Text = "Before Esc d" });
                visualRoot.Children.Add(new TextBlock { Text = (char)27 + "d" });
                visualRoot.Children.Add(new TextBlock { Text = "After Esc d" });

                visualRoot.Measure(e.PrintableArea);
                visualRoot.Arrange(new Rect(0, 0, e.PrintableArea.Width, e.PrintableArea.Height));
                visualRoot.UpdateLayout();

                e.HasMorePages = false;
            });
        }

        private void DoInUiThread(Action action)
        {
            var wait = new AutoResetEvent(false);
            Exception exception = null;

            if (_dispatcher.CheckAccess())
            {
                try
                {
                    action();
                }
                catch (Exception e)
                {
                    exception = e;
                }
            }
            else
            {
                _dispatcher.BeginInvoke(new Action(() =>
                {
                    try
                    {
                        action();
                    }
                    catch (Exception e)
                    {
                        exception = e;
                    }
                    finally
                    {
                        wait.Set();
                    }
                }));

                wait.WaitOne();
            }

            if (exception != null)
            {
                throw exception;
            }
        }
    }
}