namespace Printing
{
    using System;
    using System.Printing;
    using System.Threading;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows.Threading;

    public class PrinterTester
    {
        private Dispatcher _dispatcher;

        public PrinterTester(Dispatcher dispatcher)
        {
            _dispatcher = dispatcher;
        }

        public void Print()
        {
            var dialog = new PrintDialog();
            var result = dialog.ShowDialog();

            if (!result.Value)
            {
                return;
            }

            var printQueue = dialog.PrintQueue;
            
            var writer = PrintQueue.CreateXpsDocumentWriter(printQueue);

            writer.Write(GetVisual((int)(printQueue.DefaultPrintTicket.PageMediaSize.Width ?? 800), (int)(printQueue.DefaultPrintTicket.PageMediaSize.Height ?? 600)));
        }

        private Visual GetVisual(int width, int height)
        {
            StackPanel visualRoot = null;

            DoInUiThread(() =>
            {
                visualRoot = new StackPanel { HorizontalAlignment = HorizontalAlignment.Center };

                visualRoot.Children.Add(new TextBlock { Text = "Before Esc d" });
                visualRoot.Children.Add(new TextBlock { Text = (char)27 + "d" });
                visualRoot.Children.Add(new TextBlock { Text = "After Esc d" });

                visualRoot.Measure(new Size(width, height));
                visualRoot.Arrange(new Rect(0, 0, width, height));
                visualRoot.UpdateLayout();
            });

            return visualRoot;
        }

        private void DoInUiThread(Action action)
        {
            var wait = new AutoResetEvent(false);
            Exception exception = null;

            if (_dispatcher.CheckAccess())
            {
                try
                {
                    action();
                }
                catch (Exception e)
                {
                    exception = e;
                }
            }
            else
            {
                _dispatcher.BeginInvoke(new Action(() =>
                {
                    try
                    {
                        action();
                    }
                    catch (Exception e)
                    {
                        exception = e;
                    }
                    finally
                    {
                        wait.Set();
                    }
                }));

                wait.WaitOne();
            }

            if (exception != null)
            {
                throw exception;
            }
        }
    }
}